@@ -2,7 +2,7 @@
 
 /* fm-desktop-icon-view.c - implementation of icon view for managing the desktop.
 
-   Copyright (C) 2000 Eazel, Inc.
+   Copyright (C) 2000, 2001 Eazel, Inc.mou
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -21,40 +21,42 @@
 
    Authors: Mike Engber <engber@eazel.com>
    	    Gene Z. Ragan <gzr@eazel.com>
+	    Miguel de Icaza <miguel@ximian.com>
 */
 
 #include <config.h>
 #include "fm-desktop-icon-view.h"
 
-#include <ctype.h>
-#include <dirent.h>
-#include <fcntl.h>
+#include <X11/Xatom.h>
 #include <bonobo/bonobo-ui-util.h>
+#include <gtk/gtkmain.h>
+#include <dirent.h>
+#include <eel/eel-glib-extensions.h>
+#include <eel/eel-gnome-extensions.h>
+#include <eel/eel-gtk-extensions.h>
+#include <eel/eel-gtk-macros.h>
+#include <eel/eel-stock-dialogs.h>
+#include <eel/eel-string.h>
+#include <eel/eel-vfs-extensions.h>
+#include <fcntl.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtkcheckmenuitem.h>
-#include <libgnome/gnome-dentry.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <libnautilus-extensions/nautilus-bonobo-extensions.h>
-#include <libnautilus-extensions/nautilus-directory-background.h>
-#include <libnautilus-extensions/nautilus-directory-notify.h>
-#include <libnautilus-extensions/nautilus-file-changes-queue.h>
-#include <libnautilus-extensions/nautilus-file-operations.h>
-#include <libnautilus-extensions/nautilus-file-utilities.h>
-#include <libnautilus-extensions/nautilus-glib-extensions.h>
-#include <libnautilus-extensions/nautilus-global-preferences.h>
-#include <libnautilus-extensions/nautilus-gnome-extensions.h>
-#include <libnautilus-extensions/nautilus-gtk-extensions.h>
-#include <libnautilus-extensions/nautilus-gtk-macros.h>
-#include <libnautilus-extensions/nautilus-link.h>
-#include <libnautilus-extensions/nautilus-metadata.h>
-#include <libnautilus-extensions/nautilus-program-choosing.h>
-#include <libnautilus-extensions/nautilus-stock-dialogs.h>
-#include <libnautilus-extensions/nautilus-string.h>
-#include <libnautilus-extensions/nautilus-trash-monitor.h>
-#include <libnautilus-extensions/nautilus-volume-monitor.h>
-#include <src/nautilus-application.h>
+#include <libnautilus-private/nautilus-bonobo-extensions.h>
+#include <libnautilus-private/nautilus-directory-background.h>
+#include <libnautilus-private/nautilus-directory-notify.h>
+#include <libnautilus-private/nautilus-file-changes-queue.h>
+#include <libnautilus-private/nautilus-file-operations.h>
+#include <libnautilus-private/nautilus-file-utilities.h>
+#include <libnautilus-private/nautilus-global-preferences.h>
+#include <libnautilus-private/nautilus-link.h>
+#include <libnautilus-private/nautilus-metadata.h>
+#include <libnautilus-private/nautilus-monitor.h>
+#include <libnautilus-private/nautilus-program-choosing.h>
+#include <libnautilus-private/nautilus-trash-monitor.h>
+#include <libnautilus-private/nautilus-volume-monitor.h>
 #include <limits.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -71,27 +73,29 @@ static const char untranslated_trash_link_name[] = N_("Trash");
 
 #define DESKTOP_BACKGROUND_POPUP_PATH_DISKS	"/popups/background/Before Zoom Items/Volume Items/Disks"
 
+/* Timeout to check the desktop directory for updates */
+#define RESCAN_TIMEOUT 4000
+
 struct FMDesktopIconViewDetails
 {
 	BonoboUIComponent *ui;
 	GList *mount_black_list;
-};
 
-typedef struct {
-	GnomeVFSResult expected_result;
-	char *uri;
-	char *target_uri;
-	GdkPoint point;
-	GnomeDesktopEntry *entry;
-} CreateLinkData;
+	/* For the desktop rescanning
+	 */
+	gulong delayed_init_signal;
+	gulong done_loading_signal;
+	guint reload_desktop_timeout;
+	gboolean pending_rescan;
+};
 
 typedef struct {
 	FMDesktopIconView *view;
 	char *mount_path;
 } MountParameters;
 
-static void     fm_desktop_icon_view_initialize                   (FMDesktopIconView      *desktop_icon_view);
-static void     fm_desktop_icon_view_initialize_class             (FMDesktopIconViewClass *klass);
+static void     fm_desktop_icon_view_init                   (FMDesktopIconView      *desktop_icon_view);
+static void     fm_desktop_icon_view_class_init             (FMDesktopIconViewClass *klass);
 static void     fm_desktop_icon_view_trash_state_changed_callback (NautilusTrashMonitor   *trash,
 								   gboolean                state,
 								   gpointer                callback_data);
@@ -102,11 +106,6 @@ static void     volume_mounted_callback                           (NautilusVolum
 static void     volume_unmounted_callback                         (NautilusVolumeMonitor  *monitor,
 								   NautilusVolume         *volume,
 								   FMDesktopIconView      *icon_view);
-static void     icon_view_create_nautilus_links                   (NautilusIconContainer  *container,
-								   const GList            *item_uris,
-								   int                     x,
-								   int                     y,
-								   FMDirectoryView        *view);
 static int      desktop_icons_compare_callback                    (NautilusIconContainer  *container,
 								   NautilusFile           *file_a,
 								   NautilusFile           *file_b,
@@ -122,10 +121,19 @@ static void     update_disks_menu                                 (FMDesktopIcon
 static void     free_volume_black_list                            (FMDesktopIconView      *view);
 static gboolean	volume_link_is_selection 			  (FMDirectoryView 	  *view);
 
+EEL_CLASS_BOILERPLATE (FMDesktopIconView,
+			      fm_desktop_icon_view,
+			      FM_TYPE_ICON_VIEW)
 
-NAUTILUS_DEFINE_CLASS_BOILERPLATE (FMDesktopIconView,
-				   fm_desktop_icon_view,
-				   FM_TYPE_ICON_VIEW)
+static char *desktop_directory;
+static time_t desktop_dir_modify_time;
+
+static void
+desktop_directory_changed_callback (gpointer callback_data)
+{
+	g_free (desktop_directory);
+	desktop_directory = nautilus_get_desktop_directory ();
+}
 
 static NautilusIconContainer *
 get_icon_container (FMDesktopIconView *icon_view)
@@ -136,6 +144,80 @@ get_icon_container (FMDesktopIconView *icon_view)
 	return NAUTILUS_ICON_CONTAINER (GTK_BIN (icon_view)->child);
 }
 
+static void
+panel_desktop_area_changed (FMDesktopIconView *icon_view)
+{
+	long *borders = NULL;
+	Atom type_returned;
+	int format_returned;
+	unsigned long items_returned;
+	unsigned long bytes_after_return;
+	NautilusIconContainer *icon_container;
+
+	g_return_if_fail (FM_IS_DESKTOP_ICON_VIEW (icon_view));
+
+	icon_container = get_icon_container (icon_view);
+
+	gdk_error_trap_push ();
+	if (XGetWindowProperty (GDK_DISPLAY (),
+				GDK_ROOT_WINDOW (),
+				gdk_x11_get_xatom_by_name ("GNOME_PANEL_DESKTOP_AREA"),
+				0 /* long_offset */, 
+				4 /* long_length */,
+				False /* delete */,
+				XA_CARDINAL,
+				&type_returned,
+				&format_returned,
+				&items_returned,
+				&bytes_after_return,
+				(unsigned char **)&borders) != Success) {
+		if (borders != NULL)
+			XFree (borders);
+		borders = NULL;
+	}
+			    
+	if (gdk_error_trap_pop ()
+	    || borders == NULL
+	    || type_returned != XA_CARDINAL
+	    || items_returned != 4
+	    || format_returned != 32) {
+		nautilus_icon_container_set_margins (icon_container,
+						     0, 0, 0, 0);
+	} else {
+		nautilus_icon_container_set_margins (icon_container,
+						     borders[0 /* left */],
+						     borders[1 /* right */],
+						     borders[2 /* top */],
+						     borders[3 /* bottom */]);
+	}
+
+	if (borders != NULL)
+		XFree (borders);
+}
+
+static GdkFilterReturn
+desktop_icon_view_property_filter (GdkXEvent *gdk_xevent,
+				   GdkEvent *event,
+				   gpointer data)
+{
+	XEvent *xevent = gdk_xevent;
+	FMDesktopIconView *icon_view;
+
+	icon_view = FM_DESKTOP_ICON_VIEW (data);
+  
+	switch (xevent->type) {
+	case PropertyNotify:
+		if (xevent->xproperty.atom == gdk_x11_get_xatom_by_name ("GNOME_PANEL_DESKTOP_AREA")) {
+			panel_desktop_area_changed (icon_view);
+		}
+		break;
+	default:
+		break;
+	}
+
+	return GDK_FILTER_CONTINUE;
+}
+
 static void
 fm_desktop_icon_view_destroy (GtkObject *object)
 {
@@ -143,28 +225,49 @@ fm_desktop_icon_view_destroy (GtkObject *object)
 
 	icon_view = FM_DESKTOP_ICON_VIEW (object);
 
+	/* Remove the property filter */
+	gdk_window_remove_filter (GDK_ROOT_PARENT (),
+				  desktop_icon_view_property_filter,
+				  icon_view);
+
+	/* Remove desktop rescan timeout. */
+	if (icon_view->details->reload_desktop_timeout != 0) {
+		gtk_timeout_remove (icon_view->details->reload_desktop_timeout);
+	}
+
+	if (icon_view->details->done_loading_signal != 0) {
+		g_signal_handler_disconnect (fm_directory_view_get_model (FM_DIRECTORY_VIEW (icon_view)),
+					     icon_view->details->done_loading_signal);
+	}
+
+	if (icon_view->details->delayed_init_signal != 0) {
+		g_signal_handler_disconnect (icon_view,
+					     icon_view->details->delayed_init_signal);
+	}
+	
 	/* Delete all of the link files. */
 	delete_all_mount_links ();
 	
-	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_HOME_URI,
+	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_HOME_URI,
 					      home_uri_changed,
 					      icon_view);
 
 	/* Clean up details */	
 	if (icon_view->details->ui != NULL) {
-		bonobo_ui_component_unset_container (icon_view->details->ui);
+		bonobo_ui_component_unset_container (icon_view->details->ui, NULL);
 		bonobo_object_unref (BONOBO_OBJECT (icon_view->details->ui));
+		icon_view->details->ui = NULL;
 	}
 	
 	free_volume_black_list (icon_view);
 	
 	g_free (icon_view->details);
 
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
-fm_desktop_icon_view_initialize_class (FMDesktopIconViewClass *klass)
+fm_desktop_icon_view_class_init (FMDesktopIconViewClass *klass)
 {
 	GtkObjectClass		*object_class;
 	FMDirectoryViewClass	*fm_directory_view_class;
@@ -199,7 +302,7 @@ fm_desktop_icon_view_handle_middle_click (NautilusIconContainer *icon_container,
 
 	/* Stop the event because we don't want anyone else dealing with it. */	
 	gdk_flush ();
-	gtk_signal_emit_stop_by_name (GTK_OBJECT(icon_container), "middle_click");
+	g_signal_stop_emission_by_name (icon_container, "middle_click");
 
 	/* build an X event to represent the middle click. */
 	x_event.type = ButtonPress;
@@ -225,7 +328,7 @@ fm_desktop_icon_view_handle_middle_click (NautilusIconContainer *icon_container,
 static void
 free_volume_black_list (FMDesktopIconView *icon_view)
 {
-	nautilus_g_list_free_deep (icon_view->details->mount_black_list);
+	eel_g_list_free_deep (icon_view->details->mount_black_list);
 	icon_view->details->mount_black_list = NULL;
 }
 
@@ -234,9 +337,11 @@ volume_in_black_list (FMDesktopIconView *icon_view,
 		      const NautilusVolume *volume)
 {
 	GList *p;
+	
+	g_return_val_if_fail (FM_IS_DESKTOP_ICON_VIEW (icon_view), TRUE);
 
 	for (p = icon_view->details->mount_black_list; p != NULL; p = p->next) {
-		if (strcmp ((char *) p->data, volume->mount_path) == 0) {
+		if (strcmp ((char *) p->data, nautilus_volume_get_mount_path (volume)) == 0) {
 			return TRUE;
 		}
 	}
@@ -244,46 +349,115 @@ volume_in_black_list (FMDesktopIconView *icon_view,
 	return FALSE;
 }
 
+
+static char *
+create_unique_volume_name (const NautilusVolume *volume)
+{
+	GnomeVFSURI *uri;
+	char *uri_path, *new_name;
+	int index;
+	char *volume_name, *original_volume_name;
+	
+	new_name = NULL;
+
+	/* Start with an index of one. If we collide, the file collided with will be the actual
+	 * number one. We will rename with the next available number.
+	 */	   
+	index = 1;
+			
+	volume_name = nautilus_volume_get_name (volume);	
+		
+	uri_path = g_strdup_printf ("%s/%s", desktop_directory, volume_name);		
+	uri = gnome_vfs_uri_new (uri_path);
+	
+	/* Check for existing filename and create a unique name. */
+	while (gnome_vfs_uri_exists (uri)) {
+		gnome_vfs_uri_unref (uri);
+		g_free (uri_path);
+		
+		index++;
+		
+		g_free (new_name);
+		new_name = g_strdup_printf ("%s (%d)", volume_name, index);
+		
+		uri_path = g_strdup_printf ("%s/%s", desktop_directory, new_name);
+		uri = gnome_vfs_uri_new (uri_path);		
+	}
+	
+	if (new_name != NULL) {
+		g_free (volume_name);
+		volume_name = new_name;	
+	}
+	
+	original_volume_name = nautilus_volume_get_name (volume);
+	if (strcmp (volume_name, original_volume_name) != 0) {
+		nautilus_volume_monitor_set_volume_name (nautilus_volume_monitor_get (),
+							 volume, volume_name);
+	}
+	g_free (original_volume_name);
+
+	gnome_vfs_uri_unref (uri);
+	g_free (uri_path);
+	
+	return volume_name;
+}
+
 static void
 create_mount_link (FMDesktopIconView *icon_view,
 		   const NautilusVolume *volume)
 {
-	char *desktop_path, *target_uri, *volume_name;
+	char *target_uri, *volume_name;
 	const char *icon_name;
 
 	if (volume_in_black_list (icon_view, volume)) {
 		return;
 	}
 	
-	/* FIXME bugzilla.eazel.com 5412: Design a comprehensive desktop mounting strategy */
-	if (!nautilus_volume_monitor_volume_is_removable (nautilus_volume_monitor_get (), volume)) {
+	/* FIXME bugzilla.gnome.org 45412: Design a comprehensive desktop mounting strategy */
+	if (!nautilus_volume_is_removable (volume)) {
 		return;
 	}
-	
+
 	/* Get icon type */
-	switch (volume->type) {
-	case NAUTILUS_VOLUME_CDDA:
-	case NAUTILUS_VOLUME_CDROM:
-		icon_name = "i-cdrom.png";
+	icon_name = "i-blockdev";
+	switch (nautilus_volume_get_device_type (volume)) {
+	case NAUTILUS_DEVICE_AUDIO_CD:
+	case NAUTILUS_DEVICE_CDROM_DRIVE:
+		icon_name = "i-cdrom";
 		break;
-		
-	case NAUTILUS_VOLUME_FLOPPY:
-		icon_name = "i-floppy.png";
+
+	case NAUTILUS_DEVICE_FLOPPY_DRIVE:
+		icon_name = "i-floppy";
+		break;
+
+	case NAUTILUS_DEVICE_JAZ_DRIVE:
+		icon_name = "i-zipdisk2";
+		break;
+
+	case NAUTILUS_DEVICE_MEMORY_STICK:
+		icon_name = "gnome-ccperiph";
+		break;
+	
+	case NAUTILUS_DEVICE_NFS:
+		icon_name = "i-nfs";
+		break;
+	
+	case NAUTILUS_DEVICE_ZIP_DRIVE:
+		icon_name = "i-zipdisk";
+		break;
+
+	case NAUTILUS_DEVICE_CAMERA:
+	case NAUTILUS_DEVICE_UNKNOWN:
 		break;
-		
-	default:
-		icon_name = "i-blockdev.png";
-		break;	
 	}
 
-	desktop_path = nautilus_get_desktop_directory ();
-	volume_name = nautilus_volume_monitor_get_volume_name (volume);	
-	target_uri =  nautilus_volume_monitor_get_target_uri (volume);
-
+	target_uri = nautilus_volume_get_target_uri (volume);
+	
+	volume_name = create_unique_volume_name (volume);
+	
 	/* Create link */
-	nautilus_link_local_create (desktop_path, volume_name, icon_name, target_uri, NAUTILUS_LINK_MOUNT);
+	nautilus_link_local_create (desktop_directory, volume_name, icon_name, target_uri, NULL, NAUTILUS_LINK_MOUNT);
 				    
-	g_free (desktop_path);
 	g_free (target_uri);
 	g_free (volume_name);
 }
@@ -307,19 +481,98 @@ home_uri_changed (gpointer callback_data)
 	update_home_link_and_delete_copies ();
 }
 
+static gboolean
+do_desktop_rescan (gpointer data)
+{
+	FMDesktopIconView *desktop_icon_view;
+	struct stat buf;
+
+	desktop_icon_view = FM_DESKTOP_ICON_VIEW (data);
+	if (desktop_icon_view->details->pending_rescan) {
+		return TRUE;
+	}
+	
+	if (stat (desktop_directory, &buf) == -1) {
+		return TRUE;
+	}
+
+	if (buf.st_ctime == desktop_dir_modify_time) {
+		return TRUE;
+	}
+
+	desktop_icon_view->details->pending_rescan = TRUE;
+
+	nautilus_directory_force_reload (
+		fm_directory_view_get_model (
+			FM_DIRECTORY_VIEW (desktop_icon_view)));
+	return TRUE;
+}
+
 static void
-fm_desktop_icon_view_initialize (FMDesktopIconView *desktop_icon_view)
+done_loading (GtkObject *DirectoryView, FMDesktopIconView *desktop_icon_view)
+{
+	struct stat buf;
+
+	desktop_icon_view->details->pending_rescan = FALSE;
+	if (stat (desktop_directory, &buf) == -1) {
+		return;
+	}
+
+	desktop_dir_modify_time = buf.st_ctime;
+}
+
+/* This function is used because the NautilusDirectory model does not
+ * exist always in the desktop_icon_view, so we wait until it has been
+ * instantiated.
+ */
+static void
+delayed_init (FMDesktopIconView *desktop_icon_view)
+{
+	/* Keep track of the load time. */
+	desktop_icon_view->details->done_loading_signal = 
+		g_signal_connect (fm_directory_view_get_model (FM_DIRECTORY_VIEW (desktop_icon_view)),
+				  "done_loading",
+				  G_CALLBACK (done_loading), desktop_icon_view);
+
+	/* Monitor desktop directory. */
+	desktop_icon_view->details->reload_desktop_timeout =
+		gtk_timeout_add (RESCAN_TIMEOUT, do_desktop_rescan, desktop_icon_view);
+
+	g_signal_handler_disconnect (desktop_icon_view,
+				     desktop_icon_view->details->delayed_init_signal);
+
+	desktop_icon_view->details->delayed_init_signal = 0;
+}
+
+static void
+fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 {
 	GList *list;
 	NautilusIconContainer *icon_container;
 	GtkAllocation *allocation;
 	GtkAdjustment *hadj, *vadj;
 
+	if (desktop_directory == NULL) {
+		eel_preferences_add_callback (NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
+					      desktop_directory_changed_callback,
+					      NULL);
+		desktop_directory_changed_callback (NULL);
+	}
+
 	icon_container = get_icon_container (desktop_icon_view);
 
 	/* Set up details */
 	desktop_icon_view->details = g_new0 (FMDesktopIconViewDetails, 1);	
 
+	/* Do a reload on the desktop if we don't have FAM, a smarter
+	 * way to keep track of the items on the desktop.
+	 */
+	if (!nautilus_monitor_active ()) {
+		desktop_icon_view->details->delayed_init_signal = g_signal_connect
+			(desktop_icon_view, "begin_loading",
+			 G_CALLBACK (delayed_init), desktop_icon_view);
+	}
+	
 	nautilus_icon_container_set_is_fixed_size (icon_container, TRUE);
 	
 	/* Set up default mount black list */
@@ -336,8 +589,8 @@ fm_desktop_icon_view_initialize (FMDesktopIconView *desktop_icon_view)
 	hadj = GTK_LAYOUT (icon_container)->hadjustment;
 	vadj = GTK_LAYOUT (icon_container)->vadjustment;
 
-	nautilus_gtk_adjustment_set_value (hadj, 0);
-	nautilus_gtk_adjustment_set_value (vadj, 0);
+	eel_gtk_adjustment_set_value (hadj, 0);
+	eel_gtk_adjustment_set_value (vadj, 0);
 
 	fm_directory_view_ignore_hidden_file_preferences
 		(FM_DIRECTORY_VIEW (desktop_icon_view));
@@ -355,49 +608,42 @@ fm_desktop_icon_view_initialize (FMDesktopIconView *desktop_icon_view)
 					     	     create_one_mount_link,
 						     desktop_icon_view);
 	
-	gtk_signal_connect (GTK_OBJECT (icon_container),
-			    "middle_click",
-			    GTK_SIGNAL_FUNC (fm_desktop_icon_view_handle_middle_click),
-			    desktop_icon_view);
+	g_signal_connect (icon_container, "middle_click",
+			  G_CALLBACK (fm_desktop_icon_view_handle_middle_click), desktop_icon_view);
 			    
-	gtk_signal_connect (GTK_OBJECT (icon_container),
-			    "compare_icons",
-			    GTK_SIGNAL_FUNC (desktop_icons_compare_callback),
-			    desktop_icon_view);
+	g_signal_connect (icon_container, "compare_icons",
+			  G_CALLBACK (desktop_icons_compare_callback), desktop_icon_view);
 
-	gtk_signal_connect (GTK_OBJECT (desktop_icon_view),
-			    "event",
-			    GTK_SIGNAL_FUNC (event_callback),
-			    desktop_icon_view);
+	g_signal_connect (desktop_icon_view, "event",
+			  G_CALLBACK (event_callback), desktop_icon_view);
+	
+	g_signal_connect_object (nautilus_trash_monitor_get (), "trash_state_changed",
+				 G_CALLBACK (fm_desktop_icon_view_trash_state_changed_callback), desktop_icon_view, 0);
+	
+	g_signal_connect_object (nautilus_volume_monitor_get (), "volume_mounted",
+				 G_CALLBACK (volume_mounted_callback), desktop_icon_view, 0);
+	
+	g_signal_connect_object (nautilus_volume_monitor_get (), "volume_unmounted",
+				 G_CALLBACK (volume_unmounted_callback), desktop_icon_view, 0);
+	
+	eel_preferences_add_callback (NAUTILUS_PREFERENCES_HOME_URI,
+				      home_uri_changed,
+				      desktop_icon_view);
 
-	gtk_signal_connect (GTK_OBJECT (nautilus_trash_monitor_get ()),
-			    "trash_state_changed",
-			    fm_desktop_icon_view_trash_state_changed_callback,
-			    desktop_icon_view);
+	/* Read out the panel desktop area and update the icon container
+	 * accordingly */
+	panel_desktop_area_changed (desktop_icon_view);
 
-	gtk_signal_connect (GTK_OBJECT (nautilus_volume_monitor_get ()),
-			    "volume_mounted",
-			    volume_mounted_callback,
-			    desktop_icon_view);
-
-	gtk_signal_connect (GTK_OBJECT (nautilus_volume_monitor_get ()),
-			    "volume_unmounted",
-			    volume_unmounted_callback,
-			    desktop_icon_view);
-			    
-	gtk_signal_connect (GTK_OBJECT (icon_container),
-			    "create_nautilus_links",
-			    GTK_SIGNAL_FUNC (icon_view_create_nautilus_links),
-			    desktop_icon_view);
-
-	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_HOME_URI, home_uri_changed,
-				  	   desktop_icon_view);
+	/* Setup the property filter */
+	gdk_window_add_filter (GDK_ROOT_PARENT (),
+			       desktop_icon_view_property_filter,
+			       desktop_icon_view);
 }
 
 static void
 new_terminal_callback (BonoboUIComponent *component, gpointer data, const char *verb)
 {
-	nautilus_gnome_open_terminal (NULL);
+	eel_gnome_open_terminal (NULL);
 }
 
 static void
@@ -406,7 +652,8 @@ change_background_callback (BonoboUIComponent *component,
 			    const char *verb)
 {
 	nautilus_launch_application_from_command 
-		("background-properties-capplet", NULL, FALSE);
+		(_("Background"),
+		 "background-properties-capplet", NULL, FALSE);
 }
 
 static void
@@ -424,7 +671,7 @@ reset_background_callback (BonoboUIComponent *component,
 			   gpointer data, 
 			   const char *verb)
 {
-	nautilus_background_reset 
+	eel_background_reset 
 		(fm_directory_view_get_background (FM_DIRECTORY_VIEW (data)));
 }
 
@@ -476,7 +723,7 @@ trash_link_is_selection (FMDirectoryView *view)
 	
 	selection = fm_directory_view_get_selection (view);
 
-	if (nautilus_g_list_exactly_one_item (selection)
+	if (eel_g_list_exactly_one_item (selection)
 	    && nautilus_file_is_nautilus_link (NAUTILUS_FILE (selection->data))) {
 		uri = nautilus_file_get_uri (NAUTILUS_FILE (selection->data));
 		/* It's probably OK that this only works for local
@@ -506,7 +753,7 @@ volume_link_is_selection (FMDirectoryView *view)
 	
 	selection = fm_directory_view_get_selection (view);
 
-	if (nautilus_g_list_exactly_one_item (selection)
+	if (eel_g_list_exactly_one_item (selection)
 	    && nautilus_file_is_nautilus_link (NAUTILUS_FILE (selection->data))) {
 		uri = nautilus_file_get_uri (NAUTILUS_FILE (selection->data));
 		/* It's probably OK that this only works for local
@@ -530,16 +777,13 @@ fm_desktop_icon_view_trash_state_changed_callback (NautilusTrashMonitor *trash_m
 						   gboolean state,
 						   gpointer callback_data)
 {
-	char *desktop_path, *path;
+	char *path;
 
-	desktop_path = nautilus_get_desktop_directory ();
-	path = nautilus_make_path (desktop_path, TRASH_LINK_NAME);
+	path = nautilus_make_path (desktop_directory, TRASH_LINK_NAME);
 
-	/* Change the XML file to have a new icon. */
-	nautilus_link_local_set_icon (path, state ? "trash-empty.png" : "trash-full.png");
+	nautilus_link_local_set_icon (path, state ? "trash-empty" : "trash-full");
 
 	g_free (path);
-	g_free (desktop_path);
 }
 
 static void
@@ -574,173 +818,32 @@ volume_unmounted_callback (NautilusVolumeMonitor *monitor,
 			   NautilusVolume *volume, 
 			   FMDesktopIconView *icon_view)
 {
-	char *link_path, *desktop_path, *volume_name;
+	char *link_path, *volume_name;
 
 	g_assert (volume != NULL);
 	
-	volume_name = nautilus_volume_monitor_get_volume_name (volume);
+	volume_name = nautilus_volume_get_name (volume);
 	if (volume_name == NULL) {
 		return;
 	}
-
-	desktop_path = nautilus_get_desktop_directory ();
-	link_path = nautilus_make_path (desktop_path, volume_name);
+	
+	link_path = nautilus_make_path (desktop_directory, volume_name);
 	unlink_and_notify (link_path);
 
 	g_free (volume_name);
-	g_free (desktop_path);
 	g_free (link_path);
 }
 
-static void
-create_link_callback (GnomeVFSAsyncHandle *handle,
-		      GnomeVFSResult result,
-		      gpointer callback_data)
-{
-	char *uri, *target_uri, *position;
-	GnomeVFSResult expected_result;
-	CreateLinkData *info;
-	GList one_item_list;
-	NautilusFile *file;
-
-	info = (CreateLinkData *) callback_data;
-	
-	uri = info->uri;
-	target_uri = info->target_uri;
-	expected_result = info->expected_result;
-
-	/* Set metadata attributes */
-	file = nautilus_file_get (uri);
-	position = g_strdup_printf ("%d,%d", info->point.x, info->point.y);
-	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_ICON_POSITION, NULL, position);
-	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_CUSTOM_ICON, NULL, info->entry->icon);
-
-	/* Notify directory that this new file has been created. */
-	one_item_list.data = uri;
-	one_item_list.next = NULL;
-	one_item_list.prev = NULL;
-	nautilus_directory_notify_files_added (&one_item_list);
-	
-	g_free (uri);
-	g_free (position);
-	g_free (target_uri);
-	gnome_desktop_entry_free (info->entry);
-	g_free (callback_data);
-}
-
-
-static void
-icon_view_create_nautilus_links (NautilusIconContainer *container, const GList *item_uris,
-			   	 int x, int y, FMDirectoryView *view)
-{
-	const GList *element;
-	char *desktop_path, *target_uri, *link_path, *link_uri, *program_path;
-	GnomeVFSURI *uri;
-	GnomeDesktopEntry *entry;
-	CreateLinkData *info;
-	GnomeVFSAsyncHandle *handle;
-	int index;
-	gboolean make_nautilus_link;
-	char *link_name, *last_slash, *message;
-	const char *icon_name;
-	
-	if (item_uris == NULL) {
-		return;
-	}
-
-	make_nautilus_link = FALSE;
-	program_path = NULL;
-	
-	desktop_path = nautilus_get_desktop_directory ();
-
-	/* Iterate through all of the URIs in the list */
-	for (element = item_uris, index = 0; element != NULL; element = element->next, index++) {
-		entry = gnome_desktop_entry_load ((char *)element->data);
-		if (entry != NULL) {
-			if (entry->terminal) {
-				/* FIXME bugzilla.eazel.com 5623: Create better text here */				
-				message = _("Nautilus does not currently support "
-					    "launchers that require a terminal.");
-				nautilus_warning_dialog (message, _("Unable to Create Link"),
-					  fm_directory_view_get_containing_window (view));
-				break;
-			}
-			
-			/* Verify that have an actual path */			
-			if (entry->exec[index][0] != '/') {
-				program_path = gnome_is_program_in_path(entry->exec[index]);
-				if (program_path != NULL) {
-					target_uri = gnome_vfs_get_uri_from_local_path (program_path);
-					g_free (program_path);
-					program_path = NULL;
-				} else {
-					/* We may have a link or other unknown url here.  Assign it to the target uri */
-					target_uri = g_strdup (entry->exec[index]);
-					make_nautilus_link = TRUE;
-				}
-			} else {
-				/* Create a URI path from the entry's executable local path */
-				target_uri = gnome_vfs_get_uri_from_local_path (entry->exec[index]);
-			}
-
-			if (target_uri != NULL) {
-				if (make_nautilus_link) {
-					/* We just punt and create a NautilusLink if we have determined
-					 * that the data will not be valid for a GnomeVFS symbolic link. */
-
-					last_slash = strrchr (target_uri, '/');
-					if (last_slash != NULL) {
-						/* Make sure that the string contains more that the final slash */
-						if (strlen (last_slash) > 1) {
-							link_name = g_strdup_printf (_("Link to %s"), last_slash + 1);
-						} else {
-							link_name = g_strdup (_("Link to Unknown"));
-						}
-					} else {
-						link_name = g_strdup_printf (_("Link to %s"), target_uri);
-					}
-
-					if (link_name != NULL) {
-						if (entry->icon != NULL) {
-							icon_name = entry->icon;
-						} else {
-							icon_name = "gnome-unknown.png";
-						}
-						nautilus_link_local_create (desktop_path, link_name, icon_name, 
-								    	    target_uri, NAUTILUS_LINK_GENERIC);
-						g_free (link_name);
-					}
-					g_free (target_uri);
-				} else {
-					/* Create an actual GnomeVFS symbolic link. */						
-					uri = gnome_vfs_uri_new (target_uri);
-					if (uri != NULL) {
-						link_path = g_strdup_printf ("%s/%s", desktop_path, entry->name);			
-						link_uri = gnome_vfs_get_uri_from_local_path (link_path);
-						
-						/* Create symbolic link */
-						info = g_malloc (sizeof (CreateLinkData));
-						info->uri = link_uri;			
-						info->target_uri = target_uri;
-						info->expected_result = GNOME_VFS_OK;
-						info->point.x = x;
-						info->point.y = y;
-						info->entry = entry;
-						
-						gnome_vfs_async_create_symbolic_link (&handle, gnome_vfs_uri_new (link_path), 
-										      target_uri, create_link_callback, info);
-															
-						g_free (link_path);
-						gnome_vfs_uri_unref (uri);
-					}
-				}
-			}
-		}
-	}
-
-	g_free (desktop_path);
-}
-
+/* update_link_and_delete_copies
+ * 
+ * Look for a particular type of link on the desktop. If the right
+ * link is there, update its target URI. Delete any extra links of
+ * that type.
+ * 
+ * @is_link_function: predicate function to test whether a link is the right type.
+ * @link_name: if non-NULL, only a link with this name is considered a match.
+ * @link_target_uri: new URI to set as link target.
+ */
 static gboolean
 update_link_and_delete_copies (gboolean (*is_link_function) (const char *path),
 			       const char *link_name,
@@ -748,24 +851,21 @@ update_link_and_delete_copies (gboolean (*is_link_function) (const char *path),
 {
 	DIR *dir;
 	gboolean found_link;
-	char *desktop_path;
 	struct dirent *dir_entry;
 	char *link_path;
 
-	desktop_path = nautilus_get_desktop_directory ();
-
-	dir = opendir (desktop_path);
+	dir = opendir (desktop_directory);
 	if (dir == NULL) {
-		g_free (desktop_path);
 		return FALSE;
 	}
 
 	found_link = FALSE;
 
 	while ((dir_entry = readdir (dir)) != NULL) {
-		link_path = nautilus_make_path (desktop_path, dir_entry->d_name);
+		link_path = nautilus_make_path (desktop_directory, dir_entry->d_name);
 		if ((* is_link_function) (link_path)) {
-			if (strcmp (dir_entry->d_name, link_name) == 0) {
+			if (!found_link &&
+			     (link_name == NULL || strcmp (dir_entry->d_name, link_name) == 0)) {
 				nautilus_link_local_set_link_uri (link_path, link_target_uri);
 				found_link = TRUE;
 			} else {
@@ -777,8 +877,6 @@ update_link_and_delete_copies (gboolean (*is_link_function) (const char *path),
 	
 	closedir (dir);
 
-	g_free (desktop_path);
-	
 	return found_link;
 }
 
@@ -790,9 +888,7 @@ update_link_and_delete_copies (gboolean (*is_link_function) (const char *path),
 static void
 update_home_link_and_delete_copies (void)
 {
-	char *desktop_path, *home_link_name, *home_uri;
-
-	desktop_path = nautilus_get_desktop_directory ();
+	char *home_link_name, *home_uri;
 
 	/* Note to translators: If it's hard to compose a good home
 	 * icon name from the user name, you can use a string without
@@ -804,19 +900,19 @@ update_home_link_and_delete_copies (void)
 	 */
 	home_link_name = g_strdup_printf (_("%s's Home"), g_get_user_name ());
 	
-	home_uri = nautilus_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
+	home_uri = eel_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
 	
 	if (!update_link_and_delete_copies (nautilus_link_local_is_home_link,
-					    home_link_name,
+					    NULL,
 					    home_uri)) {
-		nautilus_link_local_create (desktop_path,
+		nautilus_link_local_create (desktop_directory,
 					    home_link_name,
-					    "temp-home.png", 
+					    "temp-home", 
 					    home_uri,
+					    NULL,
 					    NAUTILUS_LINK_HOME);
 	}
 	
-	g_free (desktop_path);
 	g_free (home_link_name);
 	g_free (home_uri);
 }
@@ -824,22 +920,24 @@ update_home_link_and_delete_copies (void)
 static void
 update_trash_link_and_delete_copies (void)
 {
-	char *desktop_path;
 
 	/* Check for trash link */
-	if (update_link_and_delete_copies (nautilus_link_local_is_trash_link,
-					   TRASH_LINK_NAME,
-					   NAUTILUS_TRASH_URI)) {
-		return;
+	if (!update_link_and_delete_copies (nautilus_link_local_is_trash_link,
+					    TRASH_LINK_NAME,
+					    EEL_TRASH_URI)) {
+		nautilus_link_local_create (desktop_directory,
+					    TRASH_LINK_NAME,
+					    "trash-empty", 
+					    EEL_TRASH_URI,
+					    NULL,
+					    NAUTILUS_LINK_TRASH);
 	}
 
-	desktop_path = nautilus_get_desktop_directory ();
-	nautilus_link_local_create (desktop_path,
-				    TRASH_LINK_NAME,
-				    "trash-empty.png", 
-				    NAUTILUS_TRASH_URI,
-				    NAUTILUS_LINK_TRASH);
-	g_free (desktop_path);
+	/* Make sure link represents current trash state */
+	fm_desktop_icon_view_trash_state_changed_callback (nautilus_trash_monitor_get (),
+						   	   nautilus_trash_monitor_is_empty (),
+						   	   NULL);
+
 }
 
 static void
@@ -914,20 +1012,21 @@ desktop_icons_compare_callback (NautilusIconContainer *container,
 				FMDesktopIconView *icon_view)
 {
 	SortCategory category_a, category_b;
-
+	
 	category_a = get_sort_category (file_a);
 	category_b = get_sort_category (file_b);
 
-	/* Let the previous handler do the compare. */
 	if (category_a == category_b) {
-		return 0;
+		return nautilus_file_compare_for_sort 
+			(file_a, file_b, NAUTILUS_FILE_SORT_BY_DISPLAY_NAME, 
+			 fm_directory_view_should_sort_directories_first (FM_DIRECTORY_VIEW (icon_view)), 
+			 FALSE);
 	}
 
 	/* We know the answer, so prevent the other handlers
 	 * from overwriting our result.
 	 */
-	gtk_signal_emit_stop_by_name (GTK_OBJECT (container),
-				      "compare_icons");
+	g_signal_stop_emission_by_name (container, "compare_icons");
 	if (category_a < category_b) {
 		return -1;
 	} else {
@@ -941,10 +1040,10 @@ mount_parameters_new (FMDesktopIconView *view, const char *mount_path)
 	MountParameters *new_parameters;
 
 	g_assert (FM_IS_DESKTOP_ICON_VIEW (view));
-	g_assert (!nautilus_str_is_empty (mount_path)); 
+	g_assert (!eel_str_is_empty (mount_path)); 
 
 	new_parameters = g_new (MountParameters, 1);
-	gtk_object_ref (GTK_OBJECT (view));
+	g_object_ref (view);
 	new_parameters->view = view;
 	new_parameters->mount_path = g_strdup (mount_path);
 
@@ -956,13 +1055,13 @@ mount_parameters_free (MountParameters *parameters)
 {
 	g_assert (parameters != NULL);
 
-	gtk_object_unref (GTK_OBJECT (parameters->view));
+	g_object_unref (parameters->view);
 	g_free (parameters->mount_path);
 	g_free (parameters);
 }
 
 static void
-mount_parameters_free_wrapper (gpointer user_data)
+mount_parameters_free_wrapper (gpointer user_data, GClosure *closure)
 {
 	mount_parameters_free ((MountParameters *)user_data);
 }
@@ -995,14 +1094,13 @@ mount_or_unmount_removable_volume (BonoboUIComponent *component,
 static void
 update_disks_menu (FMDesktopIconView *view)
 {
-	GList *disk_list;
-	GList *element;
+	const GList *disk_list, *element;
 	guint index;
 	char *name;
 	char *command_name;
 	char *command_path;
 	NautilusVolume *volume;
-
+	
 	/* Clear any previously inserted items */
 	nautilus_bonobo_remove_menu_items_and_commands
 		(view->details->ui, DESKTOP_BACKGROUND_POPUP_PATH_DISKS);
@@ -1016,19 +1114,16 @@ update_disks_menu (FMDesktopIconView *view)
 	     element = element->next, ++index) {
 		volume = element->data;
 		
-		/* Determine human-readable name from mount path */
-		name = strrchr (volume->mount_path, '/');
-		if (name != NULL) {
-			name = name + 1;
-		} else {
-			name = volume->mount_path;
-		}
 
+		/* Determine human-readable name from mount path */
+		name = nautilus_volume_monitor_get_mount_name_for_display (nautilus_volume_monitor_get (), volume);
+		
 		nautilus_bonobo_add_numbered_toggle_menu_item 
 			(view->details->ui,
 			 DESKTOP_BACKGROUND_POPUP_PATH_DISKS,
 			 index,
 			 name);
+		g_free (name);
 
 		command_name = nautilus_bonobo_get_numbered_menu_item_command
 			(view->details->ui,
@@ -1043,16 +1138,12 @@ update_disks_menu (FMDesktopIconView *view)
 		g_free (command_path);
 
 		bonobo_ui_component_add_listener_full
-			(view->details->ui,
-			 command_name,
-			 mount_or_unmount_removable_volume,
-			 mount_parameters_new (view, volume->mount_path),
-			 mount_parameters_free_wrapper);
+			(view->details->ui, command_name,
+			 g_cclosure_new (G_CALLBACK (mount_or_unmount_removable_volume),
+					 mount_parameters_new (view, nautilus_volume_get_mount_path (volume)),
+					 mount_parameters_free_wrapper));
 		g_free (command_name);
-		
-		nautilus_volume_monitor_free_volume (volume);
 	}
-	g_list_free (disk_list);
 }
 
 static void
@@ -1064,7 +1155,7 @@ real_update_menus (FMDirectoryView *view)
 	
 	g_assert (FM_IS_DESKTOP_ICON_VIEW (view));
 
-	NAUTILUS_CALL_PARENT_CLASS (FM_DIRECTORY_VIEW_CLASS, update_menus, (view));
+	EEL_CALL_PARENT (FM_DIRECTORY_VIEW_CLASS, update_menus, (view));
 
 	desktop_view = FM_DESKTOP_ICON_VIEW (view);
 
@@ -1087,7 +1178,7 @@ real_update_menus (FMDirectoryView *view)
 		 DESKTOP_COMMAND_EMPTY_TRASH_CONDITIONAL,
 		 !include_empty_trash);
 	if (include_empty_trash) {
-		if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_CONFIRM_TRASH)) {
+		if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_CONFIRM_TRASH)) {
 			label = g_strdup (_("Empty Trash..."));
 		} else {
 			label = g_strdup (_("Empty Trash"));
@@ -1128,6 +1219,7 @@ static void
 real_merge_menus (FMDirectoryView *view)
 {
 	FMDesktopIconView *desktop_view;
+	Bonobo_UIContainer ui_container;
 	BonoboUIVerb verbs [] = {
 		BONOBO_UI_VERB ("Change Background", change_background_callback),
 		BONOBO_UI_VERB ("Empty Trash Conditional", empty_trash_callback),
@@ -1137,17 +1229,20 @@ real_merge_menus (FMDirectoryView *view)
 		BONOBO_UI_VERB_END
 	};
 
-	NAUTILUS_CALL_PARENT_CLASS (FM_DIRECTORY_VIEW_CLASS, merge_menus, (view));
+	EEL_CALL_PARENT (FM_DIRECTORY_VIEW_CLASS, merge_menus, (view));
 
 	desktop_view = FM_DESKTOP_ICON_VIEW (view);
 
 	desktop_view->details->ui = bonobo_ui_component_new ("Desktop Icon View");
+
+	ui_container = fm_directory_view_get_bonobo_ui_container (view);
 	bonobo_ui_component_set_container (desktop_view->details->ui,
-					   fm_directory_view_get_bonobo_ui_container (view));
+					   ui_container, NULL);
+	bonobo_object_release_unref (ui_container, NULL);
 	bonobo_ui_util_set_ui (desktop_view->details->ui,
 			       DATADIR,
 			       "nautilus-desktop-icon-view-ui.xml",
-			       "nautilus");
+			       "nautilus", NULL);
 	bonobo_ui_component_add_verb_list_with_data (desktop_view->details->ui, verbs, view);
 }
 
