@@ -179,7 +179,6 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 					      NAUTILUS_ICON_SIZE_SMALLER);
 	
 	rend = gtk_cell_renderer_text_new ();
-	col = gtk_tree_view_column_new ();
 	col = gtk_tree_view_column_new_with_attributes ("Icon", 
 							rend,
 							"text", 
@@ -481,7 +480,6 @@ on_row_move (GtkCList *clist,
 {
 	NautilusBookmark *bookmark;
 
-	puts (__FUNCTION__);
 	bookmark = nautilus_bookmark_copy(
 		nautilus_bookmark_list_item_at(bookmarks, old_row));
 
