@@ -42,6 +42,7 @@
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-property-bag-client.h>
 #include <bonobo/bonobo-ui-util.h>
+#include <eel/eel-debug.h>
 #include <eel/eel-gdk-extensions.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-generous-bin.h>
@@ -188,7 +189,7 @@ nautilus_window_class_init (NautilusWindowClass *klass)
 				(pixbuf, &mini_icon_pixmap, &mini_icon_mask,
 				 EEL_STANDARD_ALPHA_THRESHHOLD);
 			g_object_unref (pixbuf);
-			g_atexit (unref_mini_icon);
+			eel_debug_call_at_shutdown (unref_mini_icon);
 		}
         	g_free (filename);
 	}
@@ -530,8 +531,6 @@ set_initial_window_geometry (NautilusWindow *window)
 				          max_height_for_screen));
 }
 
-#if GNOME2_CONVERSION_COMPLETE
-
 static gboolean
 location_change_at_idle_callback (gpointer callback_data)
 {
@@ -554,11 +553,11 @@ location_change_at_idle_callback (gpointer callback_data)
 /* handle bonobo events from the throbber -- since they can come in at
    any time right in the middle of things, defer until idle */
 static void 
-throbber_location_change_request_callback (BonoboListener *listener,
-					   char *event_name, 
-					   CORBA_any *arg,
-					   CORBA_Environment *ev,
-					   gpointer callback_data)
+throbber_callback (BonoboListener *listener,
+		   const char *event_name, 
+		   const CORBA_any *arg,
+		   CORBA_Environment *ev,
+		   gpointer callback_data)
 {
 	NautilusWindow *window;
 
@@ -573,8 +572,6 @@ throbber_location_change_request_callback (BonoboListener *listener,
 	}
 }
 
-#endif
-
 /* Add a dummy menu with a "View as ..." item when we first create the
  * view_as_option_menu -- without this the menu draws empty and shrunk,
  * once we populate it it grows and forces the toolbar and all the other
@@ -732,7 +729,7 @@ nautilus_window_constructed (NautilusWindow *window)
 	location_bar_wrapper = bonobo_control_new (location_bar_box);
 	bonobo_ui_component_object_set (window->details->shell_ui,
 					"/Location Bar/Wrapper",
-					bonobo_object_corba_objref (BONOBO_OBJECT (location_bar_wrapper)),
+					BONOBO_OBJREF (location_bar_wrapper),
 					NULL);
 	bonobo_ui_component_thaw (window->details->shell_ui, NULL);
 	bonobo_object_unref (BONOBO_OBJECT (location_bar_wrapper));
@@ -745,16 +742,15 @@ nautilus_window_constructed (NautilusWindow *window)
 	window->details->ui_pending_initialize_menus_part_2 = TRUE;
 
 	/* watch for throbber location changes, too */
-	if (window->throbber != CORBA_OBJECT_NIL) {
+	if (window->details->throbber != CORBA_OBJECT_NIL) {
 		CORBA_exception_init (&ev);
-		property_bag = Bonobo_Control_getProperties (window->throbber, &ev);
+		property_bag = Bonobo_Control_getProperties (window->details->throbber, &ev);
 		if (!BONOBO_EX (&ev) && property_bag != CORBA_OBJECT_NIL) {
-#if GNOME2_CONVERSION_COMPLETE
-			window->details->throbber_location_change_request_listener_id =
-				bonobo_event_source_client_add_listener
-				(property_bag, throbber_location_change_request_callback, 
-				 "Bonobo/Property:change:location", NULL, window); 
-#endif
+			window->details->throbber_listener =
+				bonobo_event_source_client_add_listener_full
+				(property_bag,
+				 g_cclosure_new (G_CALLBACK (throbber_callback), window, NULL), 
+				 "Bonobo/Property:change:location", NULL); 
 			bonobo_object_release_unref (property_bag, NULL);	
 		}
 		CORBA_exception_free (&ev);
@@ -850,23 +846,25 @@ nautilus_window_unrealize (GtkWidget *widget)
 	/* Get rid of the throbber explicitly before it self-destructs
 	 * (which it will do when the control frame goes away.
 	 */
-	if (window->throbber != CORBA_OBJECT_NIL) {
-		CORBA_exception_init (&ev);
-		property_bag = Bonobo_Control_getProperties (window->throbber, &ev);
-		if (!BONOBO_EX (&ev) && property_bag != CORBA_OBJECT_NIL) {	
-#if GNOME2_CONVERSION_COMPLETE
-			bonobo_event_source_client_remove_listener
-				(property_bag,
-				 window->details->throbber_location_change_request_listener_id,
-				 &ev);
-#endif
-			bonobo_object_release_unref (property_bag, NULL);	
+	if (window->details->throbber != CORBA_OBJECT_NIL) {
+		if (window->details->throbber_listener != CORBA_OBJECT_NIL) {
+			CORBA_exception_init (&ev);
+
+			property_bag = Bonobo_Control_getProperties (window->details->throbber, &ev);
+			if (!BONOBO_EX (&ev) && property_bag != CORBA_OBJECT_NIL) {	
+				bonobo_event_source_client_remove_listener
+					(property_bag, window->details->throbber_listener, NULL);
+				bonobo_object_release_unref (property_bag, NULL);	
+			}
+
+			CORBA_Object_release (window->details->throbber_listener, &ev);
+			window->details->throbber_listener = CORBA_OBJECT_NIL;
+
+			CORBA_exception_free (&ev);
 		}
-		CORBA_exception_free (&ev);
 
-		bonobo_object_release_unref (window->throbber, NULL);
-
-		window->throbber = CORBA_OBJECT_NIL;
+		bonobo_object_release_unref (window->details->throbber, NULL);
+		window->details->throbber = CORBA_OBJECT_NIL;
 	}
 
 	EEL_CALL_PARENT (GTK_WIDGET_CLASS, unrealize, (widget));
@@ -1719,10 +1717,9 @@ nautilus_window_allow_stop (NautilusWindow *window, gboolean allow)
 	nautilus_bonobo_set_sensitive (window->details->shell_ui,
 				       NAUTILUS_COMMAND_STOP, allow);
 
-	if (window->throbber != CORBA_OBJECT_NIL) {
+	if (window->details->throbber != CORBA_OBJECT_NIL) {
 		CORBA_exception_init (&ev);
-		property_bag = Bonobo_Control_getProperties (window->throbber, &ev);
-
+		property_bag = Bonobo_Control_getProperties (window->details->throbber, &ev);
 		if (!BONOBO_EX (&ev) && property_bag != CORBA_OBJECT_NIL) {
 			bonobo_pbclient_set_boolean (property_bag, "throbbing", allow, &ev);
 			bonobo_object_release_unref (property_bag, NULL);
@@ -1737,7 +1734,7 @@ void
 nautilus_send_history_list_changed (void)
 {
 	g_signal_emit_by_name (nautilus_signaller_get_current (),
-			 	 "history_list_changed");
+			       "history_list_changed");
 }
 
 static void
@@ -1779,31 +1776,38 @@ add_to_history_list (NautilusBookmark *bookmark)
 	 * this is not a NautilusWindow function. Perhaps it belongs
 	 * in its own file.
 	 */
+	int i;
+	GList *l, *next;
 	static gboolean free_history_list_is_set_up;
-	int extra_count, index;
 
 	g_return_if_fail (NAUTILUS_IS_BOOKMARK (bookmark));
 
 	if (!free_history_list_is_set_up) {
-		g_atexit (free_history_list);
+		eel_debug_call_at_shutdown (free_history_list);
 		free_history_list_is_set_up = TRUE;
 	}
 
-	g_object_ref (bookmark);
-	remove_from_history_list (bookmark);
-	history_list = g_list_prepend (history_list, bookmark);
+/*	g_warning ("Add to history list '%s' '%s'",
+		   nautilus_bookmark_get_name (bookmark),
+		   nautilus_bookmark_get_uri (bookmark)); */
 
-	extra_count = g_list_length (history_list) - MAX_HISTORY_ITEMS;
-	if (extra_count > 0) {
-		history_list = g_list_reverse (history_list);
-		for (index = 0; index < extra_count; ++index) {
-			g_object_unref (history_list->data);
-			history_list = g_list_remove (history_list, history_list->data);
+	if (!history_list ||
+	    nautilus_bookmark_compare_uris (history_list->data, bookmark)) {
+		g_object_ref (bookmark);
+		remove_from_history_list (bookmark);
+		history_list = g_list_prepend (history_list, bookmark);
+
+		for (i = 0, l = history_list; l; l = next) {
+			next = l->next;
+			
+			if (i++ >= MAX_HISTORY_ITEMS) {
+				g_object_unref (l->data);
+				history_list = g_list_delete_link (history_list, l);
+			}
 		}
-		history_list = g_list_reverse (history_list);
-	}
 
-	nautilus_send_history_list_changed ();
+		nautilus_send_history_list_changed ();
+	}
 }
 
 void
@@ -2197,7 +2201,7 @@ nautilus_window_get_ui_container (NautilusWindow *window)
 {
 	g_return_val_if_fail (NAUTILUS_IS_WINDOW (window), CORBA_OBJECT_NIL);
 
-	return bonobo_object_corba_objref (BONOBO_OBJECT (window->details->ui_container));
+	return BONOBO_OBJREF (window->details->ui_container);
 }
 
 void
